/*
 * Copyright 2007-2017 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "result.h"
#include "setup/exs_init_setup.h"
#include "setup/exs_qcreate_setup.h"
#include "setup/exs_qmodify_setup.h"

#define QMODIFY_TESTS 6

static char *all_qmodify_tests[QMODIFY_TESTS] = {
	"exs_qmodify(EXS_QATTR_WAIT) [no exs_init]",
	"exs_qmodify(INVALID_QHANDLE, EXS_QATTR_WAIT, attr_value, attr_length)",
	"exs_qmodify(qh, INVALID_ATTR_TYPE, attr_value, attr_length)",
	"exs_qmodify(qh, EXS_QATTR_WAIT, attr_value, 0) [attr_length does not match]",
	"exs_qmodify(qh, EXS_QATTR_WAIT, INVALID_ATTR_VALUE, attr_length)",
	"exs_qmodify(EXS_QATTR_WAIT)"
};

static void *test_qmodify(void *test_void_ptr)
{
	param *p = (param *) test_void_ptr;
	int test = p->test_number;

	/* EPERM: exs_init has not yet been called */
	if (test == 1) {
		p->test_value = "EPERM";
		p->test_name = all_qmodify_tests[test - 1];
		setup_fault_catcher(p, 0);

		int ret =
		    exs_qmodify(p->qh, EXS_QATTR_WAIT, &p->qattr_wait,
				sizeof(int));

		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EPERM);
	}

	/* EINVAL: qhandle is invalid */
	else if (test == 2) {
		p->test_value = "EINVAL";
		p->test_name = all_qmodify_tests[test - 1];
		setup_fault_catcher(p, 0);

		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);

		int ret =
		    exs_qmodify(NULL, EXS_QATTR_WAIT, &p->qattr_wait,
				sizeof(int));

		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* EINVAL: attr_type is invalid */
	else if (test == 3) {
		p->test_value = "EINVAL";
		p->test_name = all_qmodify_tests[test - 1];
		setup_fault_catcher(p, 0);

		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);

		int ret =
		    exs_qmodify(p->qh, -1, &p->qattr_wait, sizeof(int));

		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* EINVAL: attr_length does not match length of the expected type */
	else if (test == 4) {
		p->test_value = "EINVAL";
		p->test_name = all_qmodify_tests[test - 1];
		setup_fault_catcher(p, 0);

		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);

		int ret =
		    exs_qmodify(p->qh, EXS_QATTR_WAIT, &p->qattr_wait, 0);

		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* EINVAL: attr_value is invalid */
	else if (test == 5) {
		p->test_value = "EINVAL";
		p->test_name = all_qmodify_tests[test - 1];
		setup_fault_catcher(p, 0);

		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);

		int ret =
		    exs_qmodify(p->qh, EXS_QATTR_WAIT, NULL, sizeof(int));

		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EINVAL);
	}

	/* OK_EXS_QMODIFY */
	else if (test == 6) {
		p->test_value = "OK_EXS_QMODIFY";
		p->test_name = all_qmodify_tests[test - 1];
		setup_fault_catcher(p, 0);

		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		if (ok_exs_qcreate(p) < 0)
			exit(EXIT_FAILURE);

		int ret = ok_exs_qmodify(p);

		if (ret == 0) {
			print_macro(test, p->test_value, p->test_name, ret,
				    0, 0);
		} else {
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, ENOTSUP);
		}
	}

	/** test number not valid **/
	else if (test != 0) {
		fprintf(stdout, "Invalid Test Number: %d\n", test);
	}

	return NULL;
}

int main(int argc, char *argv[])
{				//arv[1] first arg from test_all.sh
	execute(argc, argv, test_qmodify, all_qmodify_tests, QMODIFY_TESTS);
}
